#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pyDialer - "A simple and handy frontend for WvDial, fork of pyWvDial"
# Copyright (C) 2008 Giacomo Lozito
# Copyright (C) 2009 Aliaksandr Stelmachonak
# Licensed under GNU General Public License v2

import pygtk
pygtk.require('2.0')
import gobject, gtk
import time
import pyDialer.wvdial.utils as wutils

def info_panel_show(parent_window):
  if ( not InfoInterface.window ):
    info_iface = InfoInterface(parent_window)
  else:
    InfoInterface.window.present()

def info_panel_is_active():
  if ( InfoInterface.window ):
    return True
  else:
    return False



def info_panel_update(info_dat):
  InfoInterface.update(info_dat)



# NOTE: do not instantiate InfoInterface directly from
# other modules; use function info_panel_show() to manage it
class InfoInterface:

  # all of these variables are used as static variables,
  # and are never used as instance variables
  window = None
  info_wid = { 'netif':None, 'lipaddr':None, 'ripaddr':None, 'pdnsaddr':None,
               'sdnsaddr':None, 'conntime_l':None, 'recbytes':None, 'sentbytes':None }
  timeout_ct_id = None
  timeout_srb_id = None
  func_srb = None
  netif_value = None

  def __init__(self, parent_window):
    
    InfoInterface.autoset_srb_function()
    
    InfoInterface.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    InfoInterface.window.set_transient_for(parent_window)
    InfoInterface.window.set_position(gtk.WIN_POS_MOUSE)
    InfoInterface.window.set_title('pyDialer - information')
    InfoInterface.window.connect("destroy", self.destroy_cb)
    InfoInterface.window.set_border_width(4)
    
    info_table = gtk.Table(10, 2, True)
    info_table.set_row_spacings(4)
    info_table.set_col_spacings(8)
    info_table.set_border_width(4)
    
    info_netif_header = self.new_info_header('Interface:')
    info_netif_label = self.new_info_label('-')
    InfoInterface.info_wid['netif'] = info_netif_label
    info_lipaddr_header = self.new_info_header('Local IP address:')
    info_lipaddr_label = self.new_info_label('-')
    InfoInterface.info_wid['lipaddr'] = info_lipaddr_label
    info_ripaddr_header = self.new_info_header('Remote IP address:')
    info_ripaddr_label = self.new_info_label('-')
    InfoInterface.info_wid['ripaddr'] = info_ripaddr_label
    info_pdnsaddr_header = self.new_info_header('Prim. DNS address:')
    info_pdnsaddr_label = self.new_info_label('-')
    InfoInterface.info_wid['pdnsaddr'] = info_pdnsaddr_label
    info_sdnsaddr_header = self.new_info_header('Sec. DNS address:')
    info_sdnsaddr_label = self.new_info_label('-')
    InfoInterface.info_wid['sdnsaddr'] = info_sdnsaddr_label
    info_conntime_header = self.new_info_header('Connection time:')
    info_conntime_label = self.new_info_label('-')
    InfoInterface.info_wid['conntime_l'] = info_conntime_label
    info_recbytes_header = self.new_info_header('Received bytes:')
    info_recbytes_label = self.new_info_label('-')
    InfoInterface.info_wid['recbytes'] = info_recbytes_label
    info_sentbytes_header = self.new_info_header('Sent bytes:')
    info_sentbytes_label = self.new_info_label('-')
    InfoInterface.info_wid['sentbytes'] = info_sentbytes_label
    
    info_table.attach(info_netif_header, 0, 1, 0, 1)
    info_table.attach(info_netif_label, 1, 2, 0, 1)
    info_table.attach(info_lipaddr_header, 0, 1, 1, 2)
    info_table.attach(info_lipaddr_label, 1, 2, 1, 2)
    info_table.attach(info_ripaddr_header, 0, 1, 2, 3)
    info_table.attach(info_ripaddr_label, 1, 2, 2, 3)
    info_table.attach(info_pdnsaddr_header, 0, 1, 3, 4)
    info_table.attach(info_pdnsaddr_label, 1, 2, 3, 4)
    info_table.attach(info_sdnsaddr_header, 0, 1, 4, 5)
    info_table.attach(info_sdnsaddr_label, 1, 2, 4, 5)
    info_table.attach(gtk.HSeparator(), 0, 2, 5, 6)
    info_table.attach(info_conntime_header, 0, 1, 6, 7)
    info_table.attach(info_conntime_label, 1, 2, 6, 7)
    info_table.attach(gtk.HSeparator(), 0, 2, 7, 8)
    info_table.attach(info_recbytes_header, 0, 1, 8, 9)
    info_table.attach(info_recbytes_label, 1, 2, 8, 9)
    info_table.attach(info_sentbytes_header, 0, 1, 9, 10)
    info_table.attach(info_sentbytes_label, 1, 2, 9, 10)
    
    info_frame = gtk.Frame()
    info_frame.add(info_table)
    
    window_hbbox = gtk.HButtonBox()
    window_hbbox.set_layout(gtk.BUTTONBOX_END)
    window_bt_close = gtk.Button(stock=gtk.STOCK_CLOSE)
    window_bt_close.connect_object("clicked", gtk.Widget.destroy, InfoInterface.window)
    window_hbbox.add(window_bt_close)
    
    window_vbox = gtk.VBox(False, 4)
    window_vbox.pack_start(info_frame, True, True, 0)
    window_vbox.pack_start(window_hbbox, False, True, 0)
    InfoInterface.window.add(window_vbox)
    window_bt_close.grab_focus()
    InfoInterface.window.show_all()

  def new_info_label(self, text=None):
    label = gtk.Label()
    label.set_alignment(0, 0.5)
    if (text):
      label.set_text(text)
    label.set_selectable(True)
    return label

  def new_info_header(self, text=None):
    label = gtk.Label()
    label.set_alignment(0, 0.5)
    if (text):
      text = r'<b>' + text + r'</b>'
      label.set_markup(text)
    label.set_selectable(False)
    return label

  def destroy_cb(self, window):
    if ( InfoInterface.timeout_ct_id != None ):
      gobject.source_remove(InfoInterface.timeout_ct_id)
    if ( InfoInterface.timeout_srb_id != None ):
      gobject.source_remove(InfoInterface.timeout_srb_id)
    InfoInterface.netif_value = None
    InfoInterface.func_srb = None
    InfoInterface.window = None

  @staticmethod
  # use wutils.get_sentrecbytes_from_sysfs for linux kernels > 2.5
  # and fall back to wutils.get_sentrecbytes_from_procfs in any other situation
  def autoset_srb_function():
    try:
      rf = open('/proc/sys/kernel/osrelease', 'r')
    except IOError:
      pass
    else:
      release = rf.readline()
      rf.close()
      if ( release[-1] == '\n' ):
        release = release[:-1]
      rels = release.split('.')
      try:
        reln = (int(rels[0]), int(rels[1]))
      except TypeError:
        pass
      else:
        if ( reln > (2, 5) ):
          InfoInterface.func_srb = staticmethod(wutils.get_sentrecbytes_from_sysfs)
          return
    InfoInterface.func_srb = staticmethod(wutils.get_sentrecbytes_from_procfs)
    return

  @staticmethod
  def update(info_dat):
    if ( InfoInterface.window != None ):
      for key, val in info_dat.iteritems():
        if ( key in InfoInterface.info_wid ):
          if ( val ):
            InfoInterface.info_wid[key].set_text(val)
          else:
            InfoInterface.info_wid[key].set_text('-')
      # store netif_value for certain purposes (such as sent/received bytes handling)
      if ( 'netif' in info_dat ):
        InfoInterface.netif_value = info_dat['netif']
      # handle connection time
      if ( 'conntime' in info_dat ) and ( info_dat['conntime'] != None ):
        conntime = info_dat['conntime']
        if ( InfoInterface.timeout_ct_id != None ):
          gobject.source_remove(InfoInterface.timeout_ct_id)
        InfoInterface.timeout_ct_id = gobject.timeout_add(500, InfoInterface.update_conntime, conntime)
        InfoInterface.update_conntime(conntime)
      elif ( InfoInterface.timeout_ct_id != None ):
        gobject.source_remove(InfoInterface.timeout_ct_id)
        InfoInterface.info_wid['conntime_l'].set_text('-')
      # handle sent/received bytes
      if ( InfoInterface.netif_value != None ):
        if ( InfoInterface.timeout_srb_id != None ):
          gobject.source_remove(InfoInterface.timeout_srb_id)
        InfoInterface.timeout_srb_id = gobject.timeout_add(1000, InfoInterface.update_srb)
        InfoInterface.update_srb()
      elif ( InfoInterface.timeout_srb_id != None ):
        gobject.source_remove(InfoInterface.timeout_srb_id)
        InfoInterface.info_wid['recbytes'].set_text('-')
        InfoInterface.info_wid['sentbytes'].set_text('-')

  @staticmethod
  def update_conntime(conntime):
    if ( InfoInterface.window != None ):
      currtime = time.time()
      difftime = currtime - conntime # delta time in seconds
      secs = difftime % 60
      difftime = (difftime - secs) // 60 # remaining delta time in minutes
      mins = difftime % 60
      hours = (difftime - mins) // 60 # delta time in hours
      ct_str = "%i h : %i m : %i s" % (hours, mins, secs)
      InfoInterface.info_wid['conntime_l'].set_text(ct_str)
      return True
    else:
      return False

  @staticmethod
  def update_srb():
    if ( InfoInterface.window != None ) and ( InfoInterface.netif_value != None ) and ( InfoInterface.func_srb != None ):
      srb = InfoInterface.func_srb(InfoInterface.netif_value)
      if ( srb != None ):
        rx = srb[1]
        tx = srb[0]
        if ( rx < 1000000 ):
          rx_format = '%i (%.2f kB)' % (rx, (float(rx)/1000))
        else:
          rx_format = '%i (%.2f MB)' % (rx, (float(rx)/1000000))
        if ( tx < 1000000 ):
          tx_format = '%i (%.2f kB)' % (tx, (float(tx)/1000))
        else:
          tx_format = '%i (%.2f MB)' % (tx, (float(tx)/1000000))
        InfoInterface.info_wid['recbytes'].set_text(rx_format)
        InfoInterface.info_wid['sentbytes'].set_text(tx_format)
        return True
      else:
        InfoInterface.info_wid['recbytes'].set_text('-')
        InfoInterface.info_wid['sentbytes'].set_text('-')
        return False
    else:
      return False
